@@ -571,7 +571,7 @@ class GaussianDiffusion(nn.Module):
         posterior_log_variance_clipped = extract(self.posterior_log_variance_clipped, t, x_t.shape)
         return posterior_mean, posterior_variance, posterior_log_variance_clipped
 
-    def model_predictions(self, x, t, x_self_cond = None, clip_x_start = False):
+    def model_predictions(self, x, t, x_self_cond = None, clip_x_start = False, rederive_pred_noise = False):
         model_output = self.model(x, t, x_self_cond)
         maybe_clip = partial(torch.clamp, min = -1., max = 1.) if clip_x_start else identity
 
@@ -580,6 +580,9 @@ class GaussianDiffusion(nn.Module):
             x_start = self.predict_start_from_noise(x, t, pred_noise)
             x_start = maybe_clip(x_start)
 
+            if clip_x_start and rederive_pred_noise:
+                pred_noise = self.predict_noise_from_start(x, t, x_start)
+
         elif self.objective == 'pred_x0':
             x_start = model_output
             x_start = maybe_clip(x_start)
@@ -647,7 +650,7 @@ class GaussianDiffusion(nn.Module):
         for time, time_next in tqdm(time_pairs, desc = 'sampling loop time step'):
             time_cond = torch.full((batch,), time, device = device, dtype = torch.long)
             self_cond = x_start if self.self_condition else None
-            pred_noise, x_start, *_ = self.model_predictions(img, time_cond, self_cond, clip_x_start = True)
+            pred_noise, x_start, *_ = self.model_predictions(img, time_cond, self_cond, clip_x_start = True, rederive_pred_noise = True)
 
             if time_next < 0:
                 img = x_start
@@ -836,6 +839,7 @@ class Trainer(object):
         # model
 
         self.model = diffusion_model
+        self.channels = diffusion_model.channels
 
         # InceptionV3 for fid-score computation
 
@@ -918,7 +922,8 @@ class Trainer(object):
 
         self.step = data['step']
         self.opt.load_state_dict(data['opt'])
-        self.ema.load_state_dict(data['ema'])
+        if self.accelerator.is_main_process:
+            self.ema.load_state_dict(data["ema"])
 
         if 'version' in data:
             print(f"loading from version {data['version']}")
@@ -928,6 +933,8 @@ class Trainer(object):
 
     @torch.no_grad()
     def calculate_activation_statistics(self, samples):
+        assert exists(self.inception_v3)
+
         features = self.inception_v3(samples)[0]
         features = rearrange(features, '... 1 1 -> ...')
 
@@ -936,6 +943,10 @@ class Trainer(object):
         return mu, sigma
 
     def fid_score(self, real_samples, fake_samples):
+
+        if self.channels == 1:
+            real_samples, fake_samples = map(lambda t: rearrange(t, 'b 1 ... -> b c ...', c = 3), (real_samples, fake_samples))
+
         min_batch = min(real_samples.shape[0], fake_samples.shape[0])
         real_samples, fake_samples = map(lambda t: t[:min_batch], (real_samples, fake_samples))
 
